using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

// Open the Add References dialog and on the .NET tab add references to:
//      Microsoft.Office.Interop.PowerPoint
//      Microsoft.Office.Interop.Word
//      Office
using Word = Microsoft.Office.Interop.Word;
using Ppt = Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Core;

namespace howto_word_to_ppt
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Initialize the file names.
        private void Form1_Load(object sender, EventArgs e)
        {
            string path = System.Windows.Forms.Application.StartupPath;
            txtWordFile.Text = path + @"\Test.docx";
            txtPowerPointFile.Text = path + @"\Test.ppt";
        }

        // Create the PowerPoint presentation.
        private void btnGo_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor = Cursors.WaitCursor;

                // Get the information we need from the Word file.
                string title;
                List<string> headings;
                GetWordData(txtWordFile.Text, txtTitleStyle.Text,
                    txtHeadingStyle.Text,
                    out title, out headings);

                if (title == "")
                {
                    MessageBox.Show("Could not find title style '" + txtTitleStyle.Text + "'");
                    return;
                }
                if (headings.Count == 0)
                {
                    MessageBox.Show("Could not find heading style '" + txtHeadingStyle.Text + "'");
                    return;
                }

                // Create the PowerPoint presentation.
                CreatePpt(txtPowerPointFile.Text, title, headings);

                MessageBox.Show("Done");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                Cursor = Cursors.Default;
            }
        }

        // Get the necessary information from the Word document.
        private void GetWordData(string file_name,
            string title_style, string heading_style,
            out string title, out List<string> headings)
        {
            // Load the Word document.
            // Get the Word application object.
            Word._Application word_app = new Word.ApplicationClass();

            object save_changes = false;
            object missing = System.Reflection.Missing.Value;

            try
            {
                // Make Word visible (optional).
                word_app.Visible = false;

                // Open the file.
                object filename = file_name;
                object confirm_conversions = false;
                object read_only = true;
                object add_to_recent_files = false;
                object format = 0;

                Word._Document word_doc =
                    word_app.Documents.Open(ref filename, ref confirm_conversions,
                        ref read_only, ref add_to_recent_files,
                        ref missing, ref missing, ref missing, ref missing,
                        ref missing, ref format, ref missing, ref missing,
                        ref missing, ref missing, ref missing, ref missing);

                try
                {
                    // Get the title.
                    object style_name_obj = txtTitleStyle.Text;
                    WordFindWordWithStyle(word_doc, word_app, style_name_obj, out title);
                    Console.WriteLine("Title: " + title);

                    // Get the headings.
                    style_name_obj = txtHeadingStyle.Text;
                    headings = new List<string>();

                    // Repeat until we find a match before the previous one.
                    int last_pos = -1;
                    for (; ; )
                    {
                        string heading;
                        int pos = WordFindWordWithStyle(word_doc, word_app, style_name_obj, out heading);
                        if ((pos == 0) || (pos < last_pos)) break;

                        last_pos = pos;
                        headings.Add(heading);
                        Console.WriteLine("Section: " + pos + ": " + heading);
                    }
                }
                catch
                {
                    // Rethrow the exception.
                    throw;
                }
                finally
                {
                    // Close the document without prompting.
                    word_doc.Close(ref save_changes, ref missing, ref missing);
                }
            }
            catch
            {
                // Rethrow the exception.
                throw;
            }
            finally
            {
                word_app.Quit(ref save_changes, ref missing, ref missing);
            }
        }

        // Find an occurrence of a style in a Word document.
        // If the style isn't found, set text = "" and return 0.
        private int WordFindWordWithStyle(Word._Document word_doc,
            Word._Application word_app, object style_name, out string text)
        {
            word_app.Selection.Find.ClearFormatting();
            object style;
            try
            {
                style = word_doc.Styles.get_Item(ref style_name);
            }
            catch
            {
                text = "";
                return 0;
            }
            word_app.Selection.Find.set_Style(ref style);

            object blank_text = "";
            object false_obj = false;
            object true_obj = true;
            object wrap_obj = Word.WdFindWrap.wdFindContinue;
            object missing = System.Reflection.Missing.Value;
            word_app.Selection.Find.Execute(ref blank_text,
                ref false_obj, ref false_obj, ref false_obj,
                ref false_obj, ref false_obj, ref true_obj,
                ref wrap_obj, ref true_obj, ref missing,
                ref missing, ref false_obj, ref false_obj,
                ref false_obj, ref false_obj);

            text = word_app.Selection.Text.Trim();
            return word_app.Selection.Start;
        }

        // Create the PowerPoint presentation.
        private void CreatePpt(string file_name,
            string title, List<string> headings)
        {
            // Get the PowerPoint application object.
            Ppt._Application ppt_app = new Ppt.ApplicationClass();

            try
            {
                // Create the presentation.
                Ppt.Presentation ppt_pres =
                    ppt_app.Presentations.Add(MsoTriState.msoFalse);
                try
                {
                    // Create the title slide.
                    Ppt.Slide title_slide =
                        ppt_pres.Slides.Add(1, PpSlideLayout.ppLayoutTitleOnly);
                    title_slide.Shapes[1].TextFrame.TextRange.Text = title;

                    // Keep track of the other slides so we can make an agenda.
                    string agenda = "";

                    // Create other slides.
                    int slide_num = 1;
                    foreach (string heading in headings)
                    {
                        slide_num++;
                        Ppt.Slide heading_slide = ppt_pres.Slides.Add(
                            slide_num, PpSlideLayout.ppLayoutText);
                        heading_slide.Shapes[1].TextFrame.TextRange.Text = heading;

                        agenda += '\n' + heading;
                    }

                    // Create the agenda.
                    if (agenda.Length > 0)
                    {
                        // Remove the initial \n.
                        agenda = agenda.Substring(1);

                        // Create the agenda slide.
                        Ppt.Slide agenda_slide = ppt_pres.Slides.Add(
                            2, PpSlideLayout.ppLayoutText);
                        agenda_slide.Shapes[1].TextFrame.TextRange.Text = "Agenda";
                        agenda_slide.Shapes[2].TextFrame.TextRange.Text = agenda;

                        // Make each line in the agenda text be a hyperlink to the
                        // corresponding section page.
                        for (int i = 1; i < slide_num; i++)
                        {
                            // Page number (the current slide number) is the section
                            // number plus 2 (for the title slide and the agenda).
                            int page_number = i + 2;

                            // Set the SubAddress.
                            // (Address gives the file or URL. We don't use Address
                            // so the destination is within this document.
                            // SubAddress gives the location within the file.
                            // In this case, that's the slide number.)
                            agenda_slide.Shapes[2].TextFrame.TextRange.Lines(i, i).
                                ActionSettings[PpMouseActivation.ppMouseClick].
                                Hyperlink.SubAddress = page_number.ToString();
                        }
                    }

                    // Save the presentation.
                    ppt_pres.SaveAs(file_name,
                        PpSaveAsFileType.ppSaveAsPresentation,
                        MsoTriState.msoFalse);
                }
                catch
                {
                    // Rethrow the exception.
                    throw;
                }
                finally
                {
                    // Close.
                    ppt_pres.Close();
                }
            }
            catch
            {
                // Rethrow the exception.
                throw;
            }
            finally
            {
                ppt_app.Quit();
            }
        }

        private void btnPickWordFile_Click(object sender, EventArgs e)
        {
            ofdWordFile.FileName = txtWordFile.Text;
            if (ofdWordFile.ShowDialog() == DialogResult.OK)
                txtWordFile.Text = ofdWordFile.FileName;
        }

        private void btnPickPptFile_Click(object sender, EventArgs e)
        {
            sfdPptFile.FileName = txtPowerPointFile.Text;
            if (sfdPptFile.ShowDialog() == DialogResult.OK)
                txtPowerPointFile.Text = sfdPptFile.FileName;
        }
    }
}
